@@ -8,7 +8,7 @@
 
 # Package name      : perl5
 # Source directory  : .
-# Configuration time: Mon Mar  9 14:20:14 EST 1998
+# Configuration time: Tue Mar 31 15:51:58 EST 1998
 # Configured by     : doughera
 # Target system     : linux fractal 2.0.33 #1 tue feb 3 10:11:46 est 1998 i686 unknown 
 
@@ -30,8 +30,8 @@ afs='false'
 alignbytes='4'
 aphostname=''
 ar='ar'
-archlib='/opt/perl/lib/i686-linux-thread/5.00462'
-archlibexp='/opt/perl/lib/i686-linux-thread/5.00462'
+archlib='/opt/perl/lib/i686-linux-thread/5.00463'
+archlibexp='/opt/perl/lib/i686-linux-thread/5.00463'
 archname='i686-linux-thread'
 archobjs=''
 awk='awk'
@@ -51,7 +51,7 @@ ccdlflags='-rdynamic'
 ccflags='-D_REENTRANT -Dbool=char -DHAS_BOOL -I/usr/local/include'
 cf_by='doughera'
 cf_email='yourname@yourhost.yourplace.com'
-cf_time='Mon Mar  9 14:20:14 EST 1998'
+cf_time='Tue Mar 31 15:51:58 EST 1998'
 chgrp=''
 chmod=''
 chown=''
@@ -124,10 +124,12 @@ d_gethbyaddr='define'
 d_gethbyname='define'
 d_gethent='define'
 d_gethname='undef'
+d_gethostprotos='define'
 d_getlogin='define'
 d_getnbyaddr='define'
 d_getnbyname='define'
 d_getnent='define'
+d_getnetprotos='define'
 d_getpbyname='define'
 d_getpbynumber='define'
 d_getpent='define'
@@ -136,9 +138,11 @@ d_getpgrp2='undef'
 d_getpgrp='define'
 d_getppid='define'
 d_getprior='define'
+d_getprotoprotos='define'
 d_getsbyname='define'
 d_getsbyport='define'
 d_getsent='define'
+d_getservprotos='define'
 d_gettimeod='define'
 d_gnulibc='define'
 d_htonl='define'
@@ -150,6 +154,7 @@ d_link='define'
 d_locconv='define'
 d_lockf='define'
 d_longdbl='define'
+d_longlong='define'
 d_lstat='define'
 d_mblen='define'
 d_mbstowcs='define'
@@ -298,7 +303,6 @@ fpostype='fpos_t'
 freetype='void'
 full_csh='/bin/csh'
 full_sed='/bin/sed'
-gcc=''
 gccversion='2.7.2.3'
 gidtype='gid_t'
 glibpth='/usr/shlib  /shlib /lib/pa1.1 /usr/lib/large /lib /usr/lib /usr/lib/386 /lib/386 /lib/large /usr/lib/small /lib/small /usr/ccs/lib /usr/ucblib /usr/local/lib '
@@ -366,7 +370,7 @@ i_varhdr='stdarg.h'
 i_vfork='undef'
 incpath=''
 inews=''
-installarchlib='/opt/perl/lib/i686-linux-thread/5.00462'
+installarchlib='/opt/perl/lib/i686-linux-thread/5.00463'
 installbin='/opt/perl/bin'
 installman1dir='/opt/perl/man/man1'
 installman3dir='/opt/perl/man/man3'
@@ -396,6 +400,7 @@ lns='/bin/ln -s'
 locincpth='/usr/local/include /opt/local/include /usr/gnu/include /opt/gnu/include /usr/GNU/include /opt/GNU/include'
 loclibpth='/usr/local/lib /opt/local/lib /usr/gnu/lib /opt/gnu/lib /usr/GNU/lib /opt/GNU/lib'
 longdblsize='12'
+longlongsize='8'
 longsize='4'
 lp=''
 lpr=''
@@ -511,7 +516,7 @@ stdio_filbuf=''
 stdio_ptr='((fp)->_IO_read_ptr)'
 strings='/usr/include/string.h'
 submit=''
-subversion='62'
+subversion='63'
 sysman='/usr/man/man1'
 tail=''
 tar=''
@@ -544,5 +549,5 @@ xlibpth='/usr/lib/386 /lib/386'
 zcat=''
 zip='zip'
 PATCHLEVEL=4
-SUBVERSION=62
+SUBVERSION=63
 CONFIG=true
