@@ -30,7 +30,7 @@ struct xpvcv {
     CV *	xcv_outside;
 #ifdef USE_THREADS
     perl_mutex *xcv_mutexp;
-    struct thread *xcv_owner;	/* current owner thread */
+    struct perl_thread *xcv_owner;	/* current owner thread */
 #endif /* USE_THREADS */
     cv_flags_t	xcv_flags;
 };
