@@ -502,7 +502,7 @@ do_kv(ARGSproto)
     }
 
     /* Guess how much room we need.  hv_max may be a few too many.  Oh well. */
-    EXTEND(sp, HvMAX(hv) * (dokeys + dovalues));
+    EXTEND(SP, HvMAX(hv) * (dokeys + dovalues));
 
     PUTBACK;	/* hv_iternext and hv_iterval might clobber stack_sp */
     while (entry = realhv ? hv_iternext(hv) : avhv_iternext((AV*)hv)) {
