@@ -5,6 +5,6 @@ optimize="-O1"
 d_setregid='undef'
 d_setreuid='undef'
 case "$usemymalloc" in
-'') usemymalloc='n' ;;
+'') usemymalloc='y' ;;
 esac
 
